package com.pvuv.pvuv.util;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * 日期工具类
 */
public class DateUtil {
    public static  Date getBillDate(String time) throws Exception{
        time = time + " 00:00:00";
        DateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sf.parse(time);
    }

    //根据时间获取当月月初和月末时间
    public static Map<String,String> getMonthTime(Date dealDate){
        Map<String,String> map = new HashMap<String,String>();
        SimpleDateFormat fromFormat = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        SimpleDateFormat toFormat = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dealDate);
        calendar.add(Calendar.MONTH, 0);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        String fromDate = fromFormat.format(calendar.getTime());
        calendar.add(Calendar.MONTH, 1);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        String toDate = toFormat.format(calendar.getTime());
        map.put("fromDate",fromDate);
        map.put("toDate",toDate);
        return map;
    }

    //根据时间获取上月月初和月末时间
    public static Map<String,String> getLastMonthTime(Date dealDate){
        Map<String,String> map = new HashMap<String,String>();
        SimpleDateFormat fromFormat = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        SimpleDateFormat toFormat = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dealDate);
        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        String fromDate = fromFormat.format(calendar.getTime());
        calendar.add(Calendar.MONTH, 1);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        String toDate = toFormat.format(calendar.getTime());
        map.put("fromDate",fromDate);
        map.put("toDate",toDate);
        return map;
    }

    /**
     * 将Date类型转换为字符串
     *
     * @param date 日期类型
     * @return 日期字符串
     */
    public static String format(Date date) {
        return format(date, SystemConstant.FORMAT_DATE_YMDHMS);
    }

    /**
     * 将Date类型转换为字符串
     *
     * @param date    日期类型
     * @param pattern 字符串格式
     * @return 日期字符串
     */
    public static String format(Date date, String pattern) {
        if (date == null) {
            return "null";
        }
        if (pattern == null || pattern.equals("") || pattern.equals("null")) {
            pattern = SystemConstant.FORMAT_DATE_YMDHMS;
        }
        return new SimpleDateFormat(pattern).format(date);
    }

    /**
     * 将Date类型转换为字符串
     *
     * @param date    日期类型
     * @param pattern 字符串格式
     * @return 日期字符串
     */
    public static String formatNull(Date date, String pattern) {
        if (date == null) {
            return null;
        }
        if (pattern == null || pattern.equals("") || pattern.equals("null")) {
            pattern = SystemConstant.FORMAT_DATE_YMDHMS;
        }
        return new SimpleDateFormat(pattern).format(date);
    }

    /**
     * 将字符串转换为Date类型
     *
     * @param date 字符串类型
     * @return 日期类型
     */
    public static Date format(String date) {
        return format(date, null);
    }

    /**
     * 将字符串转换为Date类型
     *
     * @param date    字符串类型
     * @param pattern 格式
     * @return 日期类型
     */
    public static Date format(String date, String pattern) {
        if (pattern == null || pattern.equals("") || pattern.equals("null")) {
            pattern = SystemConstant.FORMAT_DATE_YMDHMS;
        }
        if (date == null || date.equals("") || date.equals("null")) {
            return new Date();
        }
        Date d = null;
        try {
            d = new SimpleDateFormat(pattern).parse(date);
        } catch (ParseException pe) {
        }
        return d;
    }

    /**
     * 得到年月日时分秒字符串
     *
     * @return 字符串
     */
    public static String getDateYmdHms() {
        Date date = new Date();
        SimpleDateFormat formatter1 = new SimpleDateFormat("yyyyMMddHHmmss");
        String now = formatter1.format(date);
        return now;
    }

    public static Date getLastMonth(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.MONTH, -1);
        date = c.getTime();
        return date;
    }

    public static Date getNextDay(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.DAY_OF_MONTH, 1);
        date = c.getTime();
        return date;
    }

    public static Date getNextHour(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.HOUR_OF_DAY, 1);
        date = c.getTime();
        return date;
    }

    public static Date getNextTenMin(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.MINUTE, 10);
        date = c.getTime();
        return date;
    }

    public static Date getNextIMinute(Date date,int i) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.MINUTE, i);
        date = c.getTime();
        return date;
    }

    public static int getPreviousMonth() {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);
        return c.get(Calendar.MONTH);
    }

    public static int getCurrentMonth() {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, 0);
        return c.get(Calendar.MONTH);

    }

    public static Date getPreviousWeek(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.DAY_OF_MONTH, -7);
        date = c.getTime();
        return date;
    }

    //获得当前月的最后一天
    public static Date getLastDayOfMonth() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        lastDate.add(Calendar.MONTH, 1);//加一个月，变为下月的1号
        lastDate.add(Calendar.DATE, -1);//减去一天，变为当月最后一天

        return strToDate(sdf.format(lastDate.getTime()), dateFormat);
    }

    //获得当前月的最后一天
    public static String getLastDayOfMonthStr() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        lastDate.add(Calendar.MONTH, 1);//加一个月，变为下月的1号
        lastDate.add(Calendar.DATE, -1);//减去一天，变为当月最后一天

        return sdf.format(lastDate.getTime());
    }
    //获得上月的第一天
    public static String getFirstDayOfPreviousMonth() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        lastDate.add(Calendar.MONTH,-1);//变为上月的1号

        lastDate.set(Calendar.HOUR_OF_DAY, 0);
        lastDate.set(Calendar.MINUTE, 0);
        lastDate.set(Calendar.SECOND, 0);
        lastDate.set(Calendar.MILLISECOND, 0);
//        lastDate.add(Calendar.DATE, -1);//减去一天，变为当月最后一天

        return sdf.format(lastDate.getTime());
    }

    //获得当上月的最后一天
    public static String getLastDayOfBeforeMonth() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        lastDate.add(Calendar.MONTH, 0);//变为当月的1号
        lastDate.add(Calendar.DATE, -1);//减去一天，变为当月最后一天

        lastDate.set(Calendar.HOUR_OF_DAY, 23);
        lastDate.set(Calendar.MINUTE, 59);
        lastDate.set(Calendar.SECOND, 59);
        lastDate.set(Calendar.MILLISECOND, 999);

        return sdf.format(lastDate.getTime());
    }

    //获得上月的第一天
    public static String getFirstDayOfBeforePreviousMonth() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        lastDate.add(Calendar.MONTH,-2);//变为上上月的1号

        lastDate.set(Calendar.HOUR_OF_DAY, 0);
        lastDate.set(Calendar.MINUTE, 0);
        lastDate.set(Calendar.SECOND, 0);
        lastDate.set(Calendar.MILLISECOND, 0);
//        lastDate.add(Calendar.DATE, -1);//减去一天，变为当月最后一天

        return sdf.format(lastDate.getTime());
    }

    //获得当上月的最后一天
    public static String getLastDayOfBeforeMonthYMD() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        lastDate.add(Calendar.MONTH, 0);//变为当月的1号
        lastDate.add(Calendar.DATE, -1);//减去一天，变为当月最后一天

        return sdf.format(lastDate.getTime());
    }

    //获取当天时间
    public static String getNowDate() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();

        return sdf.format(lastDate.getTime());

    }
    public static String getNowDatehms() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();

        return sdf.format(lastDate.getTime());

    }

    public static String getToday() {
        String dateFormat = SystemConstant.FORMAT_DATE_YYMMDD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();

        return sdf.format(lastDate.getTime());

    }

    public static String getTodayYMD() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();

        return sdf.format(lastDate.getTime());

    }
    public static String getTodaydetail() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMDHMS_STRING;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();

        return sdf.format(lastDate.getTime());

    }


    //获取当月第一天
    public static Date getFirstDayOfMonth() {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        return strToDate(sdf.format(lastDate.getTime()), dateFormat);
    }

    //获取当月第一天
    public static Date getFirstDayOfMonth(String pattern) {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        if(null != pattern && !pattern.isEmpty()) {
            dateFormat = pattern;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1);//设为当前月的1号
        return strToDate(sdf.format(lastDate.getTime()), dateFormat);
    }
    //获取当月第一天
    public static String getFirstDayOfTheMonth(int i) {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.set(Calendar.DATE, 1+i);//设为当前月的1号
        return sdf.format(lastDate.getTime());
    }

    //获取上月第一天
    public static String getFirstDayOfTheBefordeMonth(int i) {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.add(Calendar.MONTH,-1);//变为上月
        lastDate.set(Calendar.DATE, 1+i);
        return sdf.format(lastDate.getTime());
    }
    //获取本月
    public static String getFirstmonth() {
        String dateFormat = SystemConstant.FORMAT_MONTH;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.add(Calendar.MONTH,+0);
        return sdf.format(lastDate.getTime());
    }
    //获取未来一年
    public static String getEndDaymonth() {
        String dateFormat = SystemConstant.FORMAT_MONTH;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.add(Calendar.MONTH,+12);
        return sdf.format(lastDate.getTime());
    }

    //获取下个月最后一天
    public static String getEndDayOfTheNextMonth(int i) {
        String dateFormat = SystemConstant.FORMAT_DATE_YMD;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        Calendar lastDate = Calendar.getInstance();
        lastDate.add(Calendar.MONTH,+i);//
        lastDate.set(Calendar.DATE, 29);
        return sdf.format(lastDate.getTime());
    }
    //获取当天时间
    public static Date getNowTime(String dateformat) {
        Date now = new Date();
        SimpleDateFormat dateFormat = new SimpleDateFormat(dateformat);//可以方便地修改日期格式
        return strToDate(dateFormat.format(now), dateformat);

    }

    //获取当天的开始时间
    public static String getStartTime(String date){
        String dateFromFormat = SystemConstant.FORMAT_DATE_YMD;
        String dateToFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat dateFormatFrom = new SimpleDateFormat(dateFromFormat);
        SimpleDateFormat dateFormatTo = new SimpleDateFormat(dateToFormat);
        Calendar todayStart = Calendar.getInstance();
        try{
            Date parsedDate = dateFormatFrom.parse(date);

            todayStart.setTime(parsedDate);
            todayStart.set(Calendar.HOUR_OF_DAY, 0);
            todayStart.set(Calendar.MINUTE, 0);
            todayStart.set(Calendar.SECOND, 0);
            todayStart.set(Calendar.MILLISECOND, 0);

        }catch (ParseException ex){
           ex.printStackTrace();
        }

        return dateFormatTo.format(todayStart.getTime()) ;

    }

    //获取当天的开始时间
    public static String getDayStartTime(String date,String dateFromFormat, String dateToFormat){
//        String dateFromFormat = SystemConstant.FORMAT_DATE_YMD;
//        String dateToFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat dateFormatFrom = new SimpleDateFormat(dateFromFormat);
        SimpleDateFormat dateFormatTo = new SimpleDateFormat(dateToFormat);
        Calendar todayStart = Calendar.getInstance();
        try{
            Date parsedDate = dateFormatFrom.parse(date);

            todayStart.setTime(parsedDate);
            todayStart.set(Calendar.HOUR_OF_DAY, 0);
            todayStart.set(Calendar.MINUTE, 0);
            todayStart.set(Calendar.SECOND, 0);
            todayStart.set(Calendar.MILLISECOND, 0);

        }catch (ParseException ex){
            ex.printStackTrace();
        }

        return dateFormatTo.format(todayStart.getTime()) ;

    }
     //获取当天的结束时间
    public static String getEndTime(String date){
        String dateFromFormat = SystemConstant.FORMAT_DATE_YMD;
        String dateToFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat dateFormatFrom = new SimpleDateFormat(dateFromFormat);
        SimpleDateFormat dateFormatTo = new SimpleDateFormat(dateToFormat);
        Calendar todayEnd = Calendar.getInstance();
        try{
            Date parsedDate = dateFormatFrom.parse(date);
            todayEnd.setTime(parsedDate);
            todayEnd.set(Calendar.HOUR_OF_DAY, 23);
            todayEnd.set(Calendar.MINUTE, 59);
            todayEnd.set(Calendar.SECOND, 59);
            todayEnd.set(Calendar.MILLISECOND, 999);
        }catch (ParseException ex){
         ex.printStackTrace();
        }
        return dateFormatTo.format(todayEnd.getTime());
    }

    public static String getEndTime(Date date){
        String dateFromFormat = SystemConstant.FORMAT_DATE_YMD;
        String dateToFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat dateFormatFrom = new SimpleDateFormat(dateFromFormat);
        SimpleDateFormat dateFormatTo = new SimpleDateFormat(dateToFormat);
        Calendar todayEnd = Calendar.getInstance();
        try{
            todayEnd.setTime(date);
            todayEnd.set(Calendar.HOUR_OF_DAY, 23);
            todayEnd.set(Calendar.MINUTE, 59);
            todayEnd.set(Calendar.SECOND, 59);
            todayEnd.set(Calendar.MILLISECOND, 999);
        }catch (Exception ex){
            ex.printStackTrace();
        }
        return dateFormatTo.format(todayEnd.getTime());
    }

    //获取当天的结束时间
    public static String getDayEndTime(String date,String dateFromFormat, String dateToFormat){
//        String dateFromFormat = SystemConstant.FORMAT_DATE_YMD;
//        String dateToFormat = SystemConstant.FORMAT_DATE_YMDHMS;
        SimpleDateFormat dateFormatFrom = new SimpleDateFormat(dateFromFormat);
        SimpleDateFormat dateFormatTo = new SimpleDateFormat(dateToFormat);
        Calendar todayEnd = Calendar.getInstance();
        try{
            Date parsedDate = dateFormatFrom.parse(date);
            todayEnd.setTime(parsedDate);
            todayEnd.set(Calendar.HOUR_OF_DAY, 23);
            todayEnd.set(Calendar.MINUTE, 59);
            todayEnd.set(Calendar.SECOND, 59);
            todayEnd.set(Calendar.MILLISECOND, 999);
        }catch (ParseException ex){
            ex.printStackTrace();
        }
        return dateFormatTo.format(todayEnd.getTime());
    }

    public static Date getYesterday() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -1);
        return cal.getTime();
    }

    public static Map<String,String> getFirstAndLastDayOfWeek(Integer year, Integer week) {
        week=week-1;
        Map<String,String> map=new HashMap<String, String>();
        Calendar c = new GregorianCalendar();
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, Calendar.JANUARY);
        c.set(Calendar.DATE, 1);
        Calendar cal = (GregorianCalendar) c.clone();
        cal.add(Calendar.DATE, week * 7);

        c.setFirstDayOfWeek(Calendar.MONDAY);
        c.setTime(cal.getTime());
        c.set(Calendar.DAY_OF_WEEK, c.getFirstDayOfWeek());
        map.put("first_day",format(c.getTime(), SystemConstant.FORMAT_DATE_YMDHMS));
        c.set(Calendar.DAY_OF_WEEK, c.getFirstDayOfWeek()+6);
        map.put("last_day",format(c.getTime(), SystemConstant.FORMAT_DATE_YMDHMS));
        return map;
    }
    public static String getYesterdayStr() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -1);
        String yesterday = new SimpleDateFormat(SystemConstant.FORMAT_DATE_YMD).format(cal.getTime());
        return yesterday;
    }

    public static String getYesterdayStrHDS() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -1);
        String yesterday = new SimpleDateFormat(SystemConstant.FORMAT_DATE_YMDHMS_STRING).format(cal.getTime());
        return yesterday;
    }
    public static String getYesterdayStr(int i) {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -1-i);
        String yesterday = new SimpleDateFormat(SystemConstant.FORMAT_DATE_YMD).format(cal.getTime());
        return yesterday;
    }


    public static String getHistoryByNumStr(int i) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.HOUR_OF_DAY,0);
        cal.set(Calendar.MINUTE,0);
        cal.set(Calendar.SECOND,0);
        cal.add(Calendar.DATE, -1 - i);
        String yesterday = new SimpleDateFormat(SystemConstant.FORMAT_DATE_YMDHMS).format(cal.getTime());
        return yesterday;
    }

    public static Date getElevenFifthEveryDayCheck(int hour,int minute) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.HOUR_OF_DAY,hour);
        cal.set(Calendar.MINUTE,minute);
        cal.set(Calendar.SECOND, 0);
        return cal.getTime();
    }

    public static Date getJDEveryDayCheck() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.HOUR_OF_DAY,14);
        cal.set(Calendar.MINUTE,30);
        cal.set(Calendar.SECOND, 0);
        return cal.getTime();
    }

    public static Date getLastWeekSunday(Date date) {
        Date a = DateUtils.addDays(date, -1);
        Calendar cal = Calendar.getInstance();
        cal.setTime(a);
        cal.set(Calendar.DAY_OF_WEEK, 1);

        return cal.getTime();
    }

    public static Date getJDEveryDayCheckEnd() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.HOUR_OF_DAY,15);
        cal.set(Calendar.MINUTE,30);
        cal.set(Calendar.SECOND, 0);
        return cal.getTime();
    }

    public static String getDateBefore(String date) {
        Date dates = format(date, SystemConstant.FORMAT_DATE_YMDHMS);
        Calendar cal = Calendar.getInstance();
        cal.setTime(dates);
        cal.add(Calendar.DATE, -1);
        String yesterday = new SimpleDateFormat(SystemConstant.FORMAT_DATE_YMD).format(cal.getTime());
        return yesterday;
    }

    public static Date getDateBefore(Date date, int hour) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.HOUR_OF_DAY, hour);
        return cal.getTime();
    }

    public static Date computeDate(Date date, int diffDay) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.DAY_OF_MONTH, diffDay);
        date = c.getTime();
        return date;
    }

    public static String dateToString(Date d) {
        return dateToString(d, SystemConstant.FORMAT_DATE_YMD);
    }

    public static String dateToString(Date d, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(d);
    }

    public static Date strToDate(String dateStr) {
        return strToDate(dateStr, "MM/dd/yyyy");
    }

    public static Date strParseDate(String dateStr) {
        return strParseDate(dateStr, "yyyy-MM-dd");
    }

    public static Date strToDate(String dateStr, String format) {
        if (StringUtils.isEmpty(dateStr)) {
            return null;
        }
        SimpleDateFormat dateFmt = new SimpleDateFormat(format);

        dateFmt.setLenient(false);
        ParsePosition pos = new ParsePosition(0);
        return dateFmt.parse(dateStr, pos);
    }

    public static Date strParseDate(String dateStr, String format) {
        if (StringUtils.isEmpty(dateStr)) {
            return null;
        }
        SimpleDateFormat dateFmt = new SimpleDateFormat(format);

        try {
            return dateFmt.parse(dateStr);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static boolean compareDate(Date date1, Date date2){
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        return simpleDateFormat.format(date1).equals(simpleDateFormat.format(date2));
    }

    public static String getDayOfWeek(Date date){
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);
        String DayOfWeek = "";
        switch(dayOfWeek){
            case 1:{DayOfWeek = "Sun";break;}
            case 2:{DayOfWeek = "Mon";break;}
            case 3:{DayOfWeek = "Tue";break;}
            case 4:{DayOfWeek = "Wed";break;}
            case 5:{DayOfWeek = "Thu";break;}
            case 6:{DayOfWeek = "Fri";break;}
            case 7:{DayOfWeek = "Sat";break;}
        }
        return DayOfWeek;
    }

    public static String getDayOfWeek(String date){
        String DayOfWeek = "week";
        try{
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Calendar c = Calendar.getInstance();
        c.setTime(format.parse(date));
        int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);

        switch(dayOfWeek){
            case 1:{DayOfWeek = "Sun";break;}
            case 2:{DayOfWeek = "Mon";break;}
            case 3:{DayOfWeek = "Tue";break;}
            case 4:{DayOfWeek = "Wed";break;}
            case 5:{DayOfWeek = "Thu";break;}
            case 6:{DayOfWeek = "Fri";break;}
            case 7:{DayOfWeek = "Sat";break;}
        }
        }catch (Exception e){
            return DayOfWeek;
        }
        return DayOfWeek;
    }

    public static Integer getIndexOfDayOfWeek(Date date){
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);
        return dayOfWeek;
    }

    public static Integer getWeekOfYear(Date date){
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(Calendar.WEEK_OF_YEAR);
    }

    public static Integer getWeekOfYear(String date){
        Calendar c = Calendar.getInstance();
        try{
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            c.setTime(format.parse(date));
        }catch (Exception e){
           return 100;
        }
        return c.get(Calendar.WEEK_OF_YEAR);
    }

    public static Date getWeekBeginFromWk(int week){
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.WEEK_OF_YEAR, week);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);

        return calendar.getTime();
    }

    public static Date getWeekEndFromWk(int week){
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.WEEK_OF_YEAR, week);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        calendar.add(Calendar.DAY_OF_WEEK, 6);
        return calendar.getTime();
    }

    /**
     * 判断当前日期是星期几
     *
     * @param pTime 修要判断的时间
     * @return dayForWeek 判断结果
     * @Exception 发生异常
     */
    public static int dayForWeek(String pTime) {
        int dayForWeek = 100;
        try{
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Calendar c = Calendar.getInstance();
            c.setTime(format.parse(pTime));
            if(c.get(Calendar.DAY_OF_WEEK) == 1){
                dayForWeek = 7;
            }else{
                dayForWeek = c.get(Calendar.DAY_OF_WEEK) - 1;
            }
        }catch (Exception ex){
            return dayForWeek;
        }
        return dayForWeek;
    }


    public static Date getWeekBegin() {
        Calendar cal = Calendar.getInstance();

        int day_of_week = cal.get(Calendar.DAY_OF_WEEK) - 2;

        cal.add(Calendar.DATE, -day_of_week);

        return cal.getTime();
    }

    public static Date getWeekEnd() {
        Calendar cal = Calendar.getInstance();

        int day_of_week = cal.get(Calendar.DAY_OF_WEEK) - 2;

        cal.add(Calendar.DATE, -day_of_week);

        cal.add(Calendar.DATE, 6);

        return cal.getTime();
    }

    public static Date getLastDayBegin() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY,0);
        calendar.set(Calendar.MINUTE,0);
        calendar.set(Calendar.SECOND,0);
        calendar.set(Calendar.MILLISECOND,0);
        return calendar.getTime();
    }

    public static Date getLastDayEnd() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY,23);
        calendar.set(Calendar.MINUTE,59);
        calendar.set(Calendar.SECOND,59);
        calendar.set(Calendar.MILLISECOND,999);
        return calendar.getTime();
    }

    public static Date getLastWeekBegin() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.WEEK_OF_YEAR, -1);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        return calendar.getTime();

    }

    public static Date getLastMonthBegin() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        return calendar.getTime();

    }

    public static Date getLastYearBegin() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.YEAR, -1);
        calendar.set(Calendar.MONTH, Calendar.JANUARY);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        return calendar.getTime();
    }

    public static Date getYearBegin() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MONTH, Calendar.JANUARY);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        return calendar.getTime();
    }

    public static Date getLastYearYesterday() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.YEAR, -1);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        return calendar.getTime();
    }


    public static Date getLastWeekEnd() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.WEEK_OF_YEAR, -1);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        System.out.println(calendar.getTime());
        calendar.add(Calendar.WEEK_OF_YEAR, 1);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
        return calendar.getTime();

    }

    public static Date getLastMonthEnd() {
        Calendar calendar = Calendar.getInstance();
//        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        return calendar.getTime();

    }

    public static int getCurrentYear() {
        Calendar calendar = Calendar.getInstance();

        return calendar.get(Calendar.YEAR);

    }

    public static int getYearByDate(Date date) {
        Calendar calendar = Calendar.getInstance();

        calendar.setTime(date);

        return calendar.get(Calendar.YEAR);

    }

    public static Date dateIncrement(Date date,int m){
        Calendar begin=Calendar.getInstance();
        begin.setTime(date);
        for(int i=0;i<m;i++){
            begin.add(Calendar.DAY_OF_YEAR, 1);
        }
        return begin.getTime();
    }

    public static Date getWeekBeginByDate(Date date){
        Calendar cal = Calendar.getInstance();

        cal.setTime(date);

        int day_of_week = cal.get(Calendar.DAY_OF_WEEK) - 2;

        cal.add(Calendar.DATE, -day_of_week);
        return cal.getTime();
    }

    public static Date getWeekEndByDate(Date date){
        Calendar cal = Calendar.getInstance();

        cal.setTime(date);

        int day_of_week = cal.get(Calendar.DAY_OF_WEEK) - 2;

        cal.add(Calendar.DATE, -day_of_week);

        cal.add(Calendar.DATE, 6);

        return cal.getTime();
    }

    public static boolean isSameDate(Date date1, Date date2) {
        if(null != date1 && null != date2){
            Calendar cal1 = Calendar.getInstance();
            cal1.setTime(date1);

            Calendar cal2 = Calendar.getInstance();
            cal2.setTime(date2);

            boolean isSameYear = cal1.get(Calendar.YEAR) == cal2
                    .get(Calendar.YEAR);
            boolean isSameMonth = isSameYear
                    && cal1.get(Calendar.MONTH) == cal2.get(Calendar.MONTH);
            boolean isSameDate = isSameMonth
                    && cal1.get(Calendar.DAY_OF_MONTH) == cal2
                    .get(Calendar.DAY_OF_MONTH);

            return isSameDate;
        }
        return false;

    }
    /**
     * JAVA获取某段时间内的所有日期字符串 格式化YYYY-MM-DD
     */
    public static List<String> findDatesStr(String startDate, String endDate) throws ParseException {
        List<String> datesStr=new ArrayList<String>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date dBegin = sdf.parse(startDate);
        Date dEnd = sdf.parse(endDate);
        List<Date> lDate = findDates(dBegin, dEnd);
        for (Date date : lDate) {
            datesStr.add(sdf.format(date));
        }
        return datesStr;
    }
    /**
     * JAVA获取某段时间内的所有日期
     */
    public static List<Date> findDates(Date dBegin, Date dEnd) {
        List lDate = new ArrayList();
        lDate.add(dBegin);
        Calendar calBegin = Calendar.getInstance();
        // 使用给定的 Date 设置此 Calendar 的时间
        calBegin.setTime(dBegin);
        Calendar calEnd = Calendar.getInstance();
        // 使用给定的 Date 设置此 Calendar 的时间
        calEnd.setTime(dEnd);
        // 测试此日期是否在指定日期之后
        while (dEnd.after(calBegin.getTime())) {
            // 根据日历的规则，为给定的日历字段添加或减去指定的时间量
            calBegin.add(Calendar.DAY_OF_MONTH, 1);
            lDate.add(calBegin.getTime());
        }
        return lDate;
    }

    /**
     * 根据传入的时间往前推4周
     * @param date
     * @return
     */
    public static Date getFourWeekEnd(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.WEEK_OF_YEAR,-4);
        cal.set(Calendar.DAY_OF_WEEK,Calendar.SUNDAY);
        cal.set(Calendar.HOUR_OF_DAY,23);
        cal.set(Calendar.MINUTE,59);
        cal.set(Calendar.SECOND, 59);
        return cal.getTime();
    }

//    java判断某个时间是否是在当前时间的七天之内

    public static boolean isLatestWeek(Date date, Date now){
        Calendar calendar = Calendar.getInstance();  //得到日历
        calendar.setTime(now);//把当前时间赋给日历
        calendar.add(Calendar.DAY_OF_MONTH, -7);  //设置为7天前
        Date before7days = calendar.getTime();   //得到7天前的时间
        if(before7days.getTime() < date.getTime()){
            return true;
        }else{
            return false;
        }
    }

    public static void main(String[] args) {
//        String exchangeStatus =  "1";;
//        String syncStatus = "2";
//        String waitConfirmExchangeTradeSql= "SELECT ceptrade.* from csexchange,ceptrade where csexchange.ordercode = ceptrade.taobaotid and csexchange.shopid= ceptrade.shopid " +
//                "and csexchange.`status`='%s' and ceptrade.syncstatus='%s' and starttime>date_sub(curdate(),interval 7 day) and csexchange.shopid=%s";
//        waitConfirmExchangeTradeSql = String.format(waitConfirmExchangeTradeSql,exchangeStatus,syncStatus,2);
        listDate("2018-01-01","2018-02-01");
        System.out.println(getLastMonthEnd());

//        String startTime = "2017-02-16";
//        Date startDate = DateUtil.format(startTime,SystemConstant.FORMAT_DATE_YMD);
//        Date now = DateUtil.format("2017-02-22 00:00:00",SystemConstant.FORMAT_DATE_YMDHMS);
//        System.out.println(startDate);
//
//
//        /**
//         * 判断日期是否在七天之内
//         */
//        System.out.println(DateUtil.isLatestWeek(startDate, now));

    }

    public static Date getOneSecondAgo(String syncStartTime, String dateFormat) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(Utils.convertStringToDate(syncStartTime,dateFormat));
        cal.add(Calendar.SECOND,-1);
        return cal.getTime();
    }

    public static Date getOneDayAgo(Date syncStartTime) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(syncStartTime);
        cal.add(Calendar.HOUR,-24);
        return cal.getTime();
    }

    /***
     *
     * @param date 时间
     * @return  cron类型的日期
     */
    public static String getCron(Date  date){
        SimpleDateFormat sdf = new SimpleDateFormat(SystemConstant.CRON_DATE_FORMAT);
        String formatTimeStr = "";
        if (date != null) {
            formatTimeStr = sdf.format(date);
        }
        return formatTimeStr;
    }

    /***
     *
     * @param cron Quartz cron的类型的日期
     * @return  Date日期
     */

    public static Date getDate(String cron) {


        if(cron == null) {
            return null;
        }

        SimpleDateFormat sdf = new SimpleDateFormat(SystemConstant.CRON_DATE_FORMAT);
        Date date = null;
        try {
            date = sdf.parse(cron);
        } catch (ParseException e) {
            return null;// 此处缺少异常处理
        }
        return date;
    }

    public static List<String> listDate(String startDate, String endDate) {
        List<String> result = new ArrayList<String>();
        Calendar c = Calendar.getInstance();
        DateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");

        //开始时间必须小于结束时间
        Date beginDate = null;
        Date endDate_ = null;
        try {
            beginDate = dateFormat1.parse(startDate);
            endDate_ = dateFormat1.parse(endDate);
            Date date = beginDate;
            result.add(format(beginDate,SystemConstant.FORMAT_DATE_YMD));
            while (!date.equals(endDate_)) {
                c.setTime(date);
                c.add(Calendar.DATE, 1); // 日期加1天
                date = c.getTime();
                result.add(format(c.getTime(),SystemConstant.FORMAT_DATE_YMD));
            }
            if (!beginDate.equals(endDate_)&&!result.contains(format(endDate_,SystemConstant.FORMAT_DATE_YMD))) {
                result.add(format(endDate_,SystemConstant.FORMAT_DATE_YMD));
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return result;
    }
    /**
     * @description: 获取前一天的截止时间
     * @author Ethan
     * @date 2019/4/8 14:35
     */
    public static Date getYesterdayEndTime() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_MONTH, -1);
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        cal.set(Calendar.MILLISECOND, 999);
        return cal.getTime();
    }
    /**
     * @description: 获取前一天的开始时间
     * @author Ethan
     * @date 2019/4/8 14:35
     */
    public static Date getYesterdayStartTime() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_MONTH, -1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }
    /**
     * @description: 时间字符串转为yyy-MM-dd HH:mm:ss格式的日期
     * @author Ethan
     * @date 2019/4/8 23:57
     */
    public static Date parseDateTime(String str) throws ParseException {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return df.parse(str);
    }

    /**
     * 获取i秒之后的时间
     * @param date
     * @param i
     * @return
     */
    public static Date getNextISecond(Date date,int i) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.SECOND, i);
        date = c.getTime();
        return date;
    }

    /**
     * 获取时间是今年第几周
     * @param date
     * @return
     */
    public static int getDateWeek(Date date){
        Calendar calendar = Calendar.getInstance();
        calendar.setFirstDayOfWeek(Calendar.MONDAY);
        calendar.setTime(date);
        return calendar.get(Calendar.WEEK_OF_YEAR);
    }

    /**
     * 获取周一
     * @param date
     * @return
     */
    public static String getMonday(Date date){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);//周一
        return format(calendar.getTime(),SystemConstant.FORMAT_DATE_YMD);
    }

    /**
     * 获取月份
     * @param date
     * @return
     */
    public static int getMonth(Date date){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.MONTH) + 1;
    }

    /**
     * 转化为美国太平洋时间
     * @param date
     * @return
     */
    public static String getUSATime(Date date) {
        Instant instant = date.toInstant();
        ZoneId zone = ZoneId.of("America/Los_Angeles");
        ZonedDateTime zonedDateTime = ZonedDateTime.ofInstant(instant, zone);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(SystemConstant.FORMAT_DATE_YMDHMS);
        return zonedDateTime.format(formatter);
    }

    /**
     * 转化为北京时间
     * @param date
     * @return
     */
    public static Date getCNTime(String date) {
        if (date == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(SystemConstant.FORMAT_DATE_YMDHMS);
        sdf.setTimeZone(TimeZone.getTimeZone("America/Los_Angeles"));
        Date date_ = null;
        try {
            date_ = sdf.parse(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        if (date_ != null) {
            Instant instant = date_.toInstant();
            ZoneId zone = ZoneId.of("Asia/Shanghai");
            ZonedDateTime zonedDateTime = ZonedDateTime.ofInstant(instant, zone);
            return Date.from(zonedDateTime.toInstant());
        }

        return null;
    }
}
