package com.pvuv.pvuv.util;

import org.apache.http.Consts;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.config.ConnectionConfig;
import org.apache.http.config.SocketConfig;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.CodingErrorAction;

/**
 * Created by villiam on 17/6/8.
 */
public class HttpClientConnectionPool {
    private static final Logger LOG = LoggerFactory.getLogger(HttpClientConnectionPool.class);

    static final int timeOut = 50 * 1000;

    static PoolingHttpClientConnectionManager connectionManager = new PoolingHttpClientConnectionManager();
    static {
        connectionManager.setMaxTotal(2000);
        connectionManager.setDefaultMaxPerRoute(1000);
        connectionManager.setDefaultSocketConfig(getSocketConfig());
        connectionManager.setDefaultConnectionConfig(getConnectionConfig());
    }

    public static CloseableHttpClient getClient(){
        return HttpClients.custom()
                .setConnectionManager(connectionManager)
                .setDefaultRequestConfig(getRequestConfig())
                .build();
    }

    private static RequestConfig getRequestConfig() {
        // 配置请求的超时设置
        return RequestConfig.custom()
                .setConnectionRequestTimeout(timeOut)
                .setConnectTimeout(timeOut).setSocketTimeout(timeOut).build();
    }

    private static SocketConfig getSocketConfig(){
        return SocketConfig.custom()
                .setTcpNoDelay(true)     //是否立即发送数据，设置为true会关闭Socket缓冲，默认为false
                .setSoReuseAddress(true) //是否可以在一个进程关闭Socket后，即使它还没有释放端口，其它进程还可以立即重用端口
                .setSoTimeout(50000)       //接收数据的等待超时时间，单位ms
                .setSoLinger(6)         //关闭Socket时，要么发送完所有数据，要么等待60s后，就关闭连接，此时socket.close()是阻塞的
                .setSoKeepAlive(true)    //开启监视TCP连接是否有效
                .build();
    }


    private static ConnectionConfig getConnectionConfig(){
        return ConnectionConfig.custom()
                .setMalformedInputAction(CodingErrorAction.IGNORE)
                .setUnmappableInputAction(CodingErrorAction.IGNORE)
                .setCharset(Consts.UTF_8)
                .build();
    }

    public static void releaseResponse(CloseableHttpResponse httpResponse){
        try {
            if(null != httpResponse){
                httpResponse.close();
            }
        }catch (IOException e){
            LOG.error("shut down response error happened: "+e.getMessage());
        }
    }

}
