package com.pvuv.pvuv.util;

import com.movitech.apollo.foundation.util.SequenceUtils;
import com.movitech.apollo.seamwork.model.Guidable;
import org.apache.commons.lang.StringUtils;
import org.codehaus.jackson.map.ObjectMapper;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;

/**
 * General utility classes<br>
 * <br>
 *
 * @created Jan 25, 2007 @ 12:14:05 PM
 * @author dzhao
 * @version 1.0
 */
public final class Utils {
    // For random string/GUID generation
    public static final String UPPER_ALPHA = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    public static final String LOWER_ALPHA = "abcdefghijklmnopqrstuvwxyz";
    public static final String ALPHA = UPPER_ALPHA + LOWER_ALPHA;
    public static final String ALPHANUMERIC = ALPHA + "0123456789";
    public static final String GUID_ALPHA = ALPHANUMERIC + "-_";
    private static final Random RANDOM = new Random();

    public static String integrityFileFolderLocation(String location) {
        if (!location.substring(location.length()-1).equals("/")) {
            return location + "/";
        }
        return location;
    }
    
    public static Double getDoubleValueOf(String doubleValue) {
        if (doubleValue != null) {
            return Double.valueOf(doubleValue);
        } else {
            return 0.00;
        }
    }

    public static Double getDoubleValueOf(Long doubleValue) {
        return getDoubleValueOf(doubleValue.toString());
    }
    /**
     * Create a random GUID of the maximum length<br>
     * First and last chars have to be alphanumeric
     *
     * @return random guid
     */
    public static final String randomProductGuid() {
        return "CK-" + randomString(ALPHANUMERIC, 1) + randomString(GUID_ALPHA, 28 - 2) + randomString(ALPHANUMERIC, 1);
    }



    /**
     * Create a random GUID of the maximum length<br>
     * First and last chars have to be alphanumeric
     *
     * @return random guid
     */
    public static final String randomGuid() {
        return randomString(ALPHANUMERIC, 1) + randomString(GUID_ALPHA, Guidable.MAX_GUID_SIZE - 2) + randomString(ALPHANUMERIC, 1);
    }
    /**
     * Create a string of random characters given an alphabet
     *
     * @param alpha
     * @param length
     * @return random string
     */
    public static final String randomString(String alpha, int length) {
        StringBuilder out = new StringBuilder();
        for (int i = 0; i < length; i++) {
            out.append(alpha.charAt(RANDOM.nextInt(alpha.length())));
        }
        return out.toString();
    }

    /**
     * Generate a number of random integer
     *
     * @param length
     * @return random string
     */
    public static final Integer randomInteger(int length) {
        return RANDOM.nextInt(length) + 1;
    }

    /**
     * Generate a string of random characters
     *
     * @param length
     * @return random string
     */
    public static final String randomString(int length) {
        return randomString(ALPHA, length);
    }
    /**
     * Create a random GUID for orders
     * @param shopAlias shop type like: TB,ES,MS
     * @param orderAlias order type like: OD, RO, CO
     * @return
     */
    public static final String randomGuid(String shopAlias, String orderAlias) {
        StringBuilder out = new StringBuilder();
        out.append(shopAlias);
        out.append(orderAlias);
        String code = SequenceUtils.getSequenceCodeByDay(out.toString(), 3);
        return code;
    }

    public static String convertDateToString(Date date, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }

    public static Date convertStringToDate(String date, String pattern) {
        if(date=="")
            return null;
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        try {
            return sdf.parse(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return new Date();
    }

    public static String formatDate(Date date) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        return sdf.format(date);
    }

    public static String removeSpecialCharacter(String targetString) {
        if (targetString.indexOf("}") > 0 || targetString.indexOf("{") > 0) {
            targetString = targetString.replaceAll("\\{", "");
            targetString = targetString.replaceAll("\\}", "");
        }
        return targetString;
    }

    public static String removeSpecialCharacterJson(String targetString) {
        if (targetString.indexOf("]") > 0 || targetString.indexOf("[") > 0) {
            targetString = targetString.replaceAll("\\[", "");
            targetString = targetString.replaceAll("\\]", "");
        }
        return targetString;
    }

    public static String replaceSpecialCharacter(String targetString) {
        if (targetString != null) {
            char[] invalidateChars = new char[]{'\t', '\r','\n', ',','"','!','@','#','$','%','^','&','*','(',')','_','+','{','}','[',']',':',';','\'','.','<','>','?','\\','|','\b'};
            if (!StringUtils.containsNone(targetString, invalidateChars)){
                for(char invalidateChar: invalidateChars) {
                    targetString = StringUtils.replaceChars(targetString, invalidateChar, ' ');
                }
            }
        }
        return targetString;
    }

    public static String convertObject2Json(Object obj) {
        ObjectMapper mapper = new ObjectMapper();
        try {
            String json =  mapper.writeValueAsString(obj);
            return json;
        } catch (IOException e) {
            e.printStackTrace();
            return "{}";
        }
    }

    private static byte[] lock = new byte[0];

    // 位数，默认是9位
    private final static long ramdomMath = 1000000000;

    public static String createTaobaotidSplit(String seed) {
        long r = 0;
        synchronized (lock) {
            r = (long) ((Math.random() + 1) * ramdomMath);
        }

        return seed+  System.currentTimeMillis() + String.valueOf(r).substring(1);
    }

    /**
     *
     * @param fileName 写入的文件名,完整地址
     * @param text 写入文本内容
     */
    public static void writerFile(String fileName,String text) {
        File file = new File(fileName);
        FileWriter fileWriter=null;
        try {
            if (!file.exists()){
                file.createNewFile();
            }
            fileWriter = new FileWriter(file,true);
            fileWriter.write(text);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            try {
                if (fileWriter !=null) {
                    fileWriter.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
